<?xml version="1.0" encoding="UTF-8"?>

<!--
    Created on : September 3, 2007, 8:55 PM
    Author     : Jody Gustafson
    Version    : 20071110
    Extends    : mpml.xsl
    Description: MPML stylesheet that implements behavior for ModPages elements.
-->

<xsl:stylesheet version="1.1" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:mp="http://www.worldtreesoftware.com/mpml"
  xmlns:html="http://www.w3.org/1999/xhtml"
  exclude-result-prefixes="mp html">

  <!-- Extends mpml.xsl -->
  <xsl:import href="mpml.xsl"/>
  
  <!--<xsl:output method="html" doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN"/>-->
  <xsl:strip-space elements="*"/>

  <!-- Global variables ($debug is defined in mpml.xsl) -->
  <xsl:variable name="borderSize">
    <xsl:choose>
      <xsl:when test="$debug">1</xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <!-- Base directory used to fix URLs in common pages -->
  <xsl:variable name="baseDir" select="//mp:page/@baseDir"/>

  <!-- This overrides the default behavior of the root element in mpml.xsl -->
  <!-- Here we're looking for a page element instead of html -->
  <xsl:template match="/mp:mpml">
    <xsl:apply-templates select="mp:page"/>
  </xsl:template>

  <!-- Entry point -->
  <xsl:template match="mp:page">
    <html xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
      <head>
        <xsl:variable name="debugTitle">
          <xsl:if test="$debug"> - Debug Mode</xsl:if>
        </xsl:variable>
        <title><xsl:value-of select="@title"/><xsl:value-of select="$debugTitle"/></title>
        <meta name="GENERATOR" content="ModPro MPML compiler version {$version}. http://www.WorldTreeSoftware.com"/>
        <xsl:apply-templates select="mp:head"/>
      </head>
     <!-- Start processing the body elements -->
     <xsl:apply-templates select="mp:body"/>
    </html>
  </xsl:template>

  <xsl:template match="mp:head[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:head[@name=$name]"/>
  </xsl:template>

  <xsl:template match="mp:head">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="mp:body[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:body[@name=$name]"/>
  </xsl:template>

  <xsl:template match="mp:body">
    <body>
      <xsl:call-template name="mp:setCommonAttributes"/>
      
      <xsl:if test="$debug">
        <xsl:call-template name="mp:debugWarning"/>
      </xsl:if>

      <!-- Start processing body child elements -->
      <xsl:apply-templates/>

      <xsl:if test="$debug">
        <xsl:call-template name="mp:debugWarning"/>
      </xsl:if>
    </body>
  </xsl:template>

  <xsl:template match="mp:section[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:section[@name=$name]"/>
  </xsl:template>

  <xsl:template match="mp:section">
    <div>
      <xsl:call-template name="mp:setCommonAttributes"/>
      <xsl:apply-templates/>
    </div>
  </xsl:template>
  
  <xsl:template match="mp:panel[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:panel[@name=$name]"/>
  </xsl:template>

  <!-- Implementation of a panel with region layout capabilities -->
  <!-- There are 5 regions: top, left, center, right, bottom -->
  <!-- Each region can have one or no panel-cell -->
  <xsl:template match="mp:panel[@layout='region']">
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentBegin"/></xsl:if>
    <table border="{$borderSize}" cellpadding="0" cellspacing="0">
      <xsl:call-template name="mp:setCommonAttributes"/>
      <!-- Determine how many columns to span -->
      <xsl:variable name="span">
        <xsl:value-of select="count(mp:panel-cell[@region='right' or @region='left' or @region='center'])"/>
      </xsl:variable>
      <xsl:if test="mp:panel-cell[@region='top']">
        <xsl:call-template name="mp:border-panel-cell">
          <xsl:with-param name="region">top</xsl:with-param>
          <xsl:with-param name="span"><xsl:value-of select="$span"/></xsl:with-param>
        </xsl:call-template>
      </xsl:if>
      <tr>
        <xsl:apply-templates select="mp:panel-cell[@region='left'][1]"/>
        <xsl:apply-templates select="mp:panel-cell[@region='center'][1]"/>
        <xsl:apply-templates select="mp:panel-cell[@region='right'][1]"/>
      </tr>
      <xsl:if test="mp:panel-cell[@region='bottom']">
        <xsl:call-template name="mp:border-panel-cell">
          <xsl:with-param name="region">bottom</xsl:with-param>
          <xsl:with-param name="span"><xsl:value-of select="$span"/></xsl:with-param>
        </xsl:call-template>
      </xsl:if>
    </table>
  </xsl:template>

  <!-- Implements a panel cell for the north and south cells of a border layout panel  -->
  <!-- $region - 'north' or 'south' -->
  <!-- $span - Number of columns to span -->
  <xsl:template name="mp:border-panel-cell">
    <xsl:param name="region"/>
    <xsl:param name="span"/>
    
    <xsl:for-each select="mp:panel-cell[@region=$region][1]">
      <tr>
        <xsl:choose>
          <xsl:when test="@ref">
            <xsl:variable name="name" select="@ref"/>
            <xsl:for-each select="//mp:panel-cell[@name=$name]">
              <td colspan="{$span}">
                <xsl:call-template name="mp:setCommonAttributes"/>
                <xsl:apply-templates/>
              </td>
            </xsl:for-each>
          </xsl:when>
          <xsl:otherwise>
            <td colspan="{$span}">
              <xsl:call-template name="mp:setCommonAttributes"/>
              <xsl:apply-templates/>
            </td>
          </xsl:otherwise>
        </xsl:choose>
      </tr>
    </xsl:for-each>
  </xsl:template>
  
  <!-- Implementation of a panel with horizontal, vertical and grid layout capabilities -->
  <!-- @layout - (optional) 'grid', 'vertical' or 'horizontal'(default) -->
  <!-- @rows - (optional) Number of rows -->
  <!-- @columns - (optional) Number of columns -->
  <!-- Any css style attributes -->
  <xsl:template match="mp:panel">
    <xsl:call-template name="mp:panel-impl">
      <xsl:with-param name="layout">
        <xsl:choose>
          <xsl:when test="string-length(@layout) != 0"><xsl:value-of select="@layout"/></xsl:when>
          <xsl:otherwise>horizontal</xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
      <xsl:with-param name="rows">
        <xsl:choose>
          <xsl:when test="string-length(@rows) != 0"><xsl:value-of select="@rows"/></xsl:when>
          <xsl:otherwise>1</xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
      <xsl:with-param name="columns">
        <xsl:choose>
          <xsl:when test="string-length(@columns) != 0"><xsl:value-of select="@columns"/></xsl:when>
          <xsl:otherwise>1</xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- Reference to a panel cell -->
  <xsl:template match="mp:panel-cell[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:panel-cell[@name=$name]"/>
  </xsl:template>

  <!-- Implementation of a panel cell -->
  <xsl:template match="mp:panel-cell">
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentBegin"/></xsl:if>
    <td>
      <xsl:call-template name="mp:setCommonAttributes"/>
      <xsl:apply-templates/>
    </td>
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentEnd"/></xsl:if>
  </xsl:template>

  <!-- Reference to an image -->
  <xsl:template match="mp:image[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:image[@name=$name]"/>
  </xsl:template>

  <!-- Implementation of an image -->
  <!-- valid attributes: @url, @label, @align, @width, @height, @fixUrl -->
  <!-- Set fixUrl="true" to prepend $baseDir to the url -->
  <xsl:template match="mp:image">
    <img border="0" src="{@url}" alt="{@label}" align="{@align}">
      <xsl:call-template name="mp:setCommonAttributes"/>
      <xsl:if test="@fixUrl = 'true'"><xsl:attribute name="src"><xsl:value-of select="concat($baseDir,@url)"/></xsl:attribute></xsl:if>
      <xsl:if test="@width"><xsl:attribute name="width"><xsl:value-of select="@width"/></xsl:attribute></xsl:if>
      <xsl:if test="@height"><xsl:attribute name="height"><xsl:value-of select="@height"/></xsl:attribute></xsl:if>
    </img>
  </xsl:template>

  <!-- Reference to a hyperlink -->
  <xsl:template match="mp:hyperlink[@ref]">
    <xsl:variable name="name" select="@ref"/>
    <xsl:apply-templates select="//mp:hyperlink[@name=$name]"/>
  </xsl:template>

  <!-- Implementation of a hyperlink -->
  <!-- valid attributes: @url, @target, @fixUrl -->
  <!-- Set fixUrl="true" to prepend $baseDir to the url -->
  <xsl:template match="mp:hyperlink">
    <a>
      <xsl:attribute name="href">
        <xsl:choose>
          <xsl:when test="starts-with(@url,'http://') or not(@fixUrl) or @fixUrl != 'true'"><xsl:value-of select="@url"/></xsl:when>
          <xsl:otherwise><xsl:value-of select="concat($baseDir,@url)"/></xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:if test="@target">
        <xsl:attribute name="target"><xsl:value-of select="@target"/></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </a>
  </xsl:template>
  
  <xsl:template name="mp:panel-impl">
    <xsl:param name="layout" select="'horizontal'"/>
    <xsl:param name="rows" select="1"/>
    <xsl:param name="columns" select="1"/>
    
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentBegin"/></xsl:if>
    <table border="{$borderSize}" cellpadding="0" cellspacing="0"><!-- rows="{$rows}" columns="{$columns}" layout="{$layout}">-->
      <xsl:call-template name="mp:setCommonAttributes"/>
      <!-- Deprecated in HTML 4.01
      <xsl:if test="@align">
        <xsl:attribute name="align"><xsl:value-of select="@align"/></xsl:attribute>
      </xsl:if>-->
      <!-- Decide how to lay out components -->
      <xsl:choose>
        <xsl:when test="$layout = 'grid'">
          <xsl:call-template name="panel-row">
            <xsl:with-param name="rowCount" select="$rows"/>
            <xsl:with-param name="colCount" select="$columns"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="$layout = 'vertical'">
          <xsl:for-each select="*">
            <tr>
              <xsl:apply-templates select="."/>
            </tr>
          </xsl:for-each>
        </xsl:when>
        <xsl:when test="$layout = 'horizontal'">
          <tr>
            <xsl:for-each select="*">
              <xsl:apply-templates select="."/>
            </xsl:for-each>
          </tr>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>ERROR: Invalid layout type '<xsl:value-of select="$layout"/>' for panel name='<xsl:value-of select="@name"/>'</xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </table>
    <xsl:if test="$debug"><xsl:call-template name="mp:debugCommentEnd"/></xsl:if>
  </xsl:template>

  <!-- Used to lay out multi-row-column panels -->
  <xsl:template name="panel-row">
    <xsl:param name="rowCount"/>
    <xsl:param name="colCount"/>
    <xsl:param name="row" select="1"/>
    <tr>
      <xsl:call-template name="panel-column">
        <xsl:with-param name="row" select="$row"/>
        <xsl:with-param name="colCount" select="$colCount"/>
      </xsl:call-template>
    </tr>
    <xsl:if test="$row &lt; $rowCount">
      <!-- recurse for each row -->
      <xsl:call-template name="panel-row">
        <xsl:with-param name="row" select="$row + 1"/>
        <xsl:with-param name="rowCount" select="$rowCount"/>
        <xsl:with-param name="colCount" select="$colCount"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- Used to lay out multi-row-column panels -->
  <xsl:template name="panel-column">
    <xsl:param name="row"/>
    <xsl:param name="colCount"/>
    <xsl:param name="col" select="1"/>
    <!-- Index into the correct node -->
    <xsl:apply-templates select="*[($colCount * ($row - 1)) + $col]"/>
    <xsl:if test="$col &lt; $colCount">
      <!-- recurse for each column -->
      <xsl:call-template name="panel-column">
        <xsl:with-param name="row" select="$row"/>
        <xsl:with-param name="col" select="$col + 1"/>
        <xsl:with-param name="colCount" select="$colCount"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- Used to set style and class attributes for elements -->
  <xsl:template name="mp:setCommonAttributes">
    <xsl:variable name="style">
      <!-- Parse size attribute -->
      <xsl:if test="@size">
        <xsl:text>width:</xsl:text><xsl:value-of select="substring-before(@size, ',')"/>;
        height:<xsl:value-of select="substring-after(@size, ',')"/>;
      </xsl:if>
      <!-- Parse position attribute -->
      <xsl:if test="@align">
        <xsl:choose>
          <xsl:when test="contains(@align, 'top')">vertical-align:top;</xsl:when>
          <xsl:when test="contains(@align, 'middle')">vertical-align:middle;</xsl:when>
          <xsl:when test="contains(@align, 'bottom')">vertical-align:bottom;</xsl:when>
        </xsl:choose>
        <xsl:choose>
          <xsl:when test="contains(@align, 'left')">float:left;text-align:left;</xsl:when>
          <xsl:when test="contains(@align, 'center')">margin-right:auto; margin-left:auto;</xsl:when>
          <xsl:when test="contains(@align, 'right')">float:right;text-align:right;</xsl:when>
        </xsl:choose>
      </xsl:if>
      <xsl:if test="@style">
        <!-- Add any explicitely defined styles -->
        <xsl:value-of select="@style"/>
      </xsl:if>
      <!-- Add every other attribute as a style attribute -->
      <xsl:for-each select="@*[not(contains('size class align name id layout rows columns style region label url fixUrl', name()))]">
        <xsl:value-of select="name()"/>:<xsl:value-of select="."/>;
      </xsl:for-each>
    </xsl:variable>

    <!-- Add id attribute using module name -->
    <xsl:if test="@name">
      <xsl:attribute name="id"><xsl:value-of select="@name"/></xsl:attribute>
    </xsl:if>

    <!-- Add style attribute if style attributes were defined -->
    <xsl:if test="string-length($style) != 0">
      <xsl:attribute name="style">
        <!-- Remove tabs and line feeds -->
        <xsl:value-of select="translate($style, '&#xA;&#9;', '')"/>
      </xsl:attribute>
    </xsl:if>
    
    <!-- Add class if specified -->
    <xsl:if test="string-length(@class) != 0">
      <xsl:attribute name="class"><xsl:value-of select="@class"/></xsl:attribute>
    </xsl:if>
  </xsl:template>
  
  <!-- A module that adds a certain amount of whitespace between elements -->
  <xsl:template match="mp:whitespace" name="whitespace">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <td>
      <xsl:call-template name="mp:setCommonAttributes"/>
      <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
    </td>
  </xsl:template>

  <!-- Debugging templates -->
  <xsl:template name="mp:debugWarning">
    <table border="1" cellpadding="0" cellspacing="0" width="100%" bgcolor="yellow">
      <tr><td border="1" cellpadding="0" cellspacing="0" width="100%" bgcolor="yellow" align="center"><strong>Debugging ON</strong></td></tr>
    </table>
  </xsl:template>
  
</xsl:stylesheet>